/*
 * Tencent is pleased to support the open source community by making TKEStack
 * available.
 *
 * Copyright (C) 2012-2023 Tencent. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use
 * this file except in compliance with the License. You may obtain a copy of the
 * License at
 *
 * https://opensource.org/licenses/Apache-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
	v1alpha1 "tkestack.io/kstone/pkg/apis/kstone/v1alpha1"
	scheme "tkestack.io/kstone/pkg/generated/clientset/versioned/scheme"
)

// EtcdInspectionsGetter has a method to return a EtcdInspectionInterface.
// A group's client should implement this interface.
type EtcdInspectionsGetter interface {
	EtcdInspections(namespace string) EtcdInspectionInterface
}

// EtcdInspectionInterface has methods to work with EtcdInspection resources.
type EtcdInspectionInterface interface {
	Create(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.CreateOptions) (*v1alpha1.EtcdInspection, error)
	Update(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.UpdateOptions) (*v1alpha1.EtcdInspection, error)
	UpdateStatus(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.UpdateOptions) (*v1alpha1.EtcdInspection, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.EtcdInspection, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.EtcdInspectionList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.EtcdInspection, err error)
	EtcdInspectionExpansion
}

// etcdInspections implements EtcdInspectionInterface
type etcdInspections struct {
	client rest.Interface
	ns     string
}

// newEtcdInspections returns a EtcdInspections
func newEtcdInspections(c *KstoneV1alpha1Client, namespace string) *etcdInspections {
	return &etcdInspections{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the etcdInspection, and returns the corresponding etcdInspection object, and an error if there is any.
func (c *etcdInspections) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.EtcdInspection, err error) {
	result = &v1alpha1.EtcdInspection{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("etcdinspections").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of EtcdInspections that match those selectors.
func (c *etcdInspections) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.EtcdInspectionList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.EtcdInspectionList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("etcdinspections").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested etcdInspections.
func (c *etcdInspections) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("etcdinspections").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a etcdInspection and creates it.  Returns the server's representation of the etcdInspection, and an error, if there is any.
func (c *etcdInspections) Create(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.CreateOptions) (result *v1alpha1.EtcdInspection, err error) {
	result = &v1alpha1.EtcdInspection{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("etcdinspections").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(etcdInspection).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a etcdInspection and updates it. Returns the server's representation of the etcdInspection, and an error, if there is any.
func (c *etcdInspections) Update(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.UpdateOptions) (result *v1alpha1.EtcdInspection, err error) {
	result = &v1alpha1.EtcdInspection{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("etcdinspections").
		Name(etcdInspection.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(etcdInspection).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *etcdInspections) UpdateStatus(ctx context.Context, etcdInspection *v1alpha1.EtcdInspection, opts v1.UpdateOptions) (result *v1alpha1.EtcdInspection, err error) {
	result = &v1alpha1.EtcdInspection{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("etcdinspections").
		Name(etcdInspection.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(etcdInspection).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the etcdInspection and deletes it. Returns an error if one occurs.
func (c *etcdInspections) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("etcdinspections").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *etcdInspections) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("etcdinspections").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched etcdInspection.
func (c *etcdInspections) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.EtcdInspection, err error) {
	result = &v1alpha1.EtcdInspection{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("etcdinspections").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
